@@ -247,16 +247,12 @@ orsf_vi_oobag_ <- function(object, type_vi, oobag_fun){
 
  }
 
- y <- as.matrix(object$data[, get_names_y(object)])
+ y <- prep_y_from_orsf(object)
+ x <- prep_x_from_orsf(object)
 
  # Put data in the same order that it was in when object was fit
  sorted <- order(y[, 1], -y[, 2])
 
- x <- as.matrix(
-  ref_code(x_data = object$data,
-           fi = get_fctr_info(object),
-           names_x_data = get_names_x(object))
- )
 
  if(is.null(oobag_fun)) {
 
@@ -283,7 +279,6 @@ orsf_vi_oobag_ <- function(object, type_vi, oobag_fun){
   "chf"  = "H"
  )
 
-
  out <- f_oobag_vi(x = x[sorted, ],
                    y = y[sorted, ],
                    forest = object$forest,
